from .core import route, use, run_server, protect, Boask
from .templates.html import html_templates
from .error import error_page, error_path
from .auth.login import login
from .auth.register import register
from .middleware.logging import logging_middleware
from .middleware.cors import cors
from .middleware.rate_limit import rate_limit
from .middleware.auth import auth_middleware
from .auth.ban import Ban
from werkzeug.wrappers import Request, Response
from werkzeug.routing import Map, Rule
from werkzeug.exceptions import HTTPException, NotFound
from werkzeug.serving import run_simple
from .version import __version__
import jwt

ban_user = Ban.ban_user 

__version__ = version.__version__

__all__ = [
    "route",
    "use",
    "protect",
    "run_server",
    "html_templates",
    "error_page",
    "error_path",
    "login",
    "register",
    "logging_middleware",
    "cors",
    "rate_limit",
    "auth_middleware",
    "Boask",
    "Request",
    "Response",
    "Map",
    "Rule",
    "HTTPException",
    "NotFound",
    "run_simple",
    "ban_user",
    "jwt"
]



import os
import re
from html import escape

TEMPLATES_DIR = os.path.join(os.getcwd(), "templates")

def _load_template(filename: str) -> str:
    path = os.path.join(TEMPLATES_DIR, filename)
    if not os.path.exists(path):
        raise FileNotFoundError(f"Boask template not found: {filename}\n   Looked in: {TEMPLATES_DIR}")
    with open(path, "r", encoding="utf-8") as f:
        return f.read()

class html_templates:
    """Simple {{ variable }} templating with automatic HTML escaping"""
    
    @staticmethod
    def render(template_name: str, **context) -> str:
        template = _load_template(template_name)
        
        def repl(match):
            key = match.group(1).strip()
            value = context.get(key, "")
            return escape(str(value))
        
        return re.sub(r'{{([^}]+)}}', repl, template)


# banlist.py
from pathlib import Path

BANLIST_FILE = Path(__file__).parent / "banned_users.txt"

def load_banned_users() -> set:
    """Load banned usernames from the banlist file."""
    if not BANLIST_FILE.exists():
        return set()
    with open(BANLIST_FILE, "r", encoding="utf-8") as f:
        return set(line.strip() for line in f if line.strip())

def is_banned(username: str) -> bool:
    """Check if a username is banned."""
    banned_users = load_banned_users()
    return username in banned_users

def ban_user(username: str) -> None:
    """Add a username to the banlist file."""
    banned_users = load_banned_users()
    if username not in banned_users:
        with open(BANLIST_FILE, "a", encoding="utf-8") as f:
            f.write(username + "\n")



import hashlib
import secrets

USERS = {}
TOKENS = {}

def hash_password(password: str) -> str:
    return hashlib.sha256(password.encode("utf-8")).hexdigest()

def verify_password(password: str, hashed: str) -> bool:
    return hash_password(password) == hashed

def create_token(username: str) -> str:
    token = secrets.token_hex(32)
    TOKENS[token] = username
    return token



from .users import USERS, verify_password, create_token
from .banlist import is_banned
from pathlib import Path
import urllib.parse
class Ban:
    @staticmethod
    def ban_user(handler):
        banlist_path = Path(__file__).parent / "banned_users.txt"
        if not banlist_path.exists():
            banlist_path.touch()
        if handler.command != "POST":
            handler.send_response(405)
            handler.end_headers()
            return b"Method Not Allowed"

        length = int(handler.headers.get("Content-Length", 0))
        body = handler.rfile.read(length).decode("utf-8")

        data = urllib.parse.parse_qs(body)
        username = data.get("username", [None])[0]
        password = data.get("password", [None])[0]

        if not username or not password:
            handler.send_response(400)
            handler.end_headers()
            return b"Missing credentials"

        if is_banned(username):
            handler.send_response(403)
            handler.end_headers()
            return b"User is banned"

        if username not in USERS:
            handler.send_response(401)
            handler.end_headers()
            return b"Invalid credentials"

        if not verify_password(password, USERS[username]):
            handler.send_response(401)
            handler.end_headers()
            return b"Invalid credentials"

        token = create_token(username)

        handler.send_response(200)
        handler.send_header("Content-Type", "application/json")
        handler.end_headers()

        return f'{{"token": "{token}"}}'.encode("utf-8")


from werkzeug.wrappers import Request, Response
from werkzeug.routing import Map, Rule
from werkzeug.exceptions import HTTPException, NotFound
from werkzeug.serving import run_simple
import os
import urllib.parse
from threading import Thread
import time

STATIC_DIR = "static"

ROUTES = []
MIDDLEWARES = []
PROTECTED_ROUTES = {}

def route(path, methods=None):
    methods = methods or ["GET"]
    def decorator(func):
        for m in methods:
            ROUTES.append((path, m.upper(), func))
        return func
    return decorator

def protect(path, methods=None, middleware=None):
    methods = methods or ["GET"]
    for m in methods:
        PROTECTED_ROUTES[(path, m.upper())] = middleware

def use(middleware_func):
    MIDDLEWARES.append(middleware_func)

class Boask:
    def __init__(self):
        self.url_map = Map([Rule(r[0], endpoint=r[2], methods=[r[1]]) for r in ROUTES])

    def wsgi_app(self, environ, start_response):
        request = Request(environ)
        path = request.path
        method = request.method.upper()

        for mw in MIDDLEWARES:
            try:
                mw(request)
            except StopIteration:
                return Response("Middleware stopped request", status=403)(environ, start_response)

        if path.startswith("/static/"):
            file_path = os.path.join(STATIC_DIR, path[len("/static/"):].lstrip("/"))
            full_path = os.path.abspath(file_path)
            if full_path.startswith(os.path.abspath(STATIC_DIR)) and os.path.isfile(full_path):
                with open(full_path, "rb") as f:
                    return Response(f.read(), content_type="application/octet-stream")(environ, start_response)
            return Response("File not found", status=404)(environ, start_response)

        if (path, method) in PROTECTED_ROUTES:
            try:
                PROTECTED_ROUTES[(path, method)](request)
            except StopIteration:
                return Response("Unauthorized", status=401)(environ, start_response)

        for r_path, r_method, r_func in ROUTES:
            if r_path == path and r_method == method:
                try:
                    response = r_func(request)
                    if isinstance(response, tuple) and len(response) == 2:
                        body, code = response
                        if isinstance(body, str):
                            body = body.encode("utf-8")
                        return Response(body, status=code)(environ, start_response)
                    if isinstance(response, str):
                        response = response.encode("utf-8")
                    return Response(response, status=200, content_type="text/html; charset=utf-8")(environ, start_response)
                except Exception as e:
                    return Response(f"Boask Error: {str(e)}", status=500)(environ, start_response)
        return Response("Not Found", status=404)(environ, start_response)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)

def run_server(port=8080, host="", debug=False):
    os.makedirs(STATIC_DIR, exist_ok=True)

    def serve():
        run_simple(host, port, Boask(), use_reloader=False)

    if debug:
        files_to_watch = []
        for root, dirs, files in os.walk(os.getcwd()):
            for f in files:
                if f.endswith(".py") or root.endswith("templates"):
                    files_to_watch.append(os.path.join(root, f))
        last_mtimes = {f: os.path.getmtime(f) for f in files_to_watch}
        while True:
            serve_thread = Thread(target=serve)
            serve_thread.start()
            try:
                while serve_thread.is_alive():
                    time.sleep(1)
                    reload_needed = False
                    for f in files_to_watch:
                        if os.path.getmtime(f) != last_mtimes[f]:
                            reload_needed = True
                            last_mtimes[f] = os.path.getmtime(f)
                    if reload_needed:
                        os._exit(3)
            except KeyboardInterrupt:
                break
    else:
        serve()



import json
from pathlib import Path
from werkzeug.security import generate_password_hash, check_password_hash
import jwt
import datetime
import secrets

USERS_FILE = Path("users.json")
BANLIST_FILE = Path("banned_users.txt")
SECRET_FILE = Path("secret.key")

if SECRET_FILE.exists():
    SECRET_KEY = SECRET_FILE.read_text().strip()
else:
    SECRET_KEY = secrets.token_hex(32)
    SECRET_FILE.write_text(SECRET_KEY)

def load_users():
    if not USERS_FILE.exists():
        return {}
    with open(USERS_FILE, "r") as f:
        return json.load(f)

def save_users(users):
    with open(USERS_FILE, "w") as f:
        json.dump(users, f)

def load_banlist():
    if not BANLIST_FILE.exists():
        BANLIST_FILE.touch()
        return set()
    with open(BANLIST_FILE, "r") as f:
        return set(line.strip() for line in f if line.strip())

def ban_user(username):
    banned = load_banlist()
    if username not in banned:
        with open(BANLIST_FILE, "a") as f:
            f.write(username + "\n")

def register_user(username, password, role="user"):
    users = load_users()
    if username in users:
        return False
    users[username] = {
        "password": generate_password_hash(password),
        "role": role
    }
    save_users(users)
    return True

def verify_password(username, password):
    users = load_users()
    if username not in users:
        return False
    return check_password_hash(users[username]["password"], password)

def create_token(username, expire_minutes=60):
    payload = {
        "sub": username,
        "exp": datetime.datetime.utcnow() + datetime.timedelta(minutes=expire_minutes)
    }
    return jwt.encode(payload, SECRET_KEY, algorithm="HS256")

def verify_token(token):
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=["HS256"])
        return payload["sub"]
    except jwt.ExpiredSignatureError:
        return None
    except jwt.InvalidTokenError:
        return None



import urllib.parse
from .users import USERS, verify_password, create_token

def login(handler):
    if handler.command != "POST":
        handler.send_response(405)
        handler.end_headers()
        return b"Method Not Allowed"

    length = int(handler.headers.get("Content-Length", 0))
    body = handler.rfile.read(length).decode("utf-8")

    data = urllib.parse.parse_qs(body)
    username = data.get("username", [None])[0]
    password = data.get("password", [None])[0]

    if not username or not password:
        handler.send_response(400)
        handler.end_headers()
        return b"Missing credentials"

    if username not in USERS:
        handler.send_response(401)
        handler.end_headers()
        return b"Invalid credentials"

    if not verify_password(password, USERS[username]):
        handler.send_response(401)
        handler.end_headers()
        return b"Invalid credentials"

    token = create_token(username)

    handler.send_response(200)
    handler.send_header("Content-Type", "application/json")
    handler.end_headers()

    return f'{{"token": "{token}"}}'.encode("utf-8")



import time

def logging_middleware(handler):
    if not hasattr(handler, "_start_time"):
        handler._start_time = time.time()
        return

    duration = (time.time() - handler._start_time) * 1000
    status = getattr(handler, "_last_status", 200)

    print(
        f"[BOASK] {handler.command} {handler.path} "
        f"-> {status} ({duration:.2f}ms)"
    )



def auth_middleware(
    token: str,
    header: str = "Authorization",
    prefix: str = "Bearer "
):
    def middleware(handler):
        value = handler.headers.get(header)

        if not value or not value.startswith(prefix):
            handler.send_response(401)
            handler.send_header("Content-Type", "text/plain")
            handler.end_headers()
            handler.wfile.write(b"Unauthorized")
            raise StopIteration

        received = value[len(prefix):].strip()
        if received != token:
            handler.send_response(403)
            handler.send_header("Content-Type", "text/plain")
            handler.end_headers()
            handler.wfile.write(b"Forbidden")
            raise StopIteration

    return middleware



import time

def rate_limit(max_requests=60, per_seconds=60):
    hits = {}

    def middleware(handler):
        ip = handler.client_address[0]
        now = time.time()

        window = hits.get(ip, [])
        window = [t for t in window if now - t < per_seconds]

        if len(window) >= max_requests:
            handler.send_response(429)
            handler.send_header("Content-Type", "text/plain")
            handler.send_header("Retry-After", str(per_seconds))
            handler.end_headers()
            handler.wfile.write(b"Too Many Requests")
            raise StopIteration

        window.append(now)
        hits[ip] = window

    return middleware



error_path = None

from .templates.html import html_templates
from .templates.tsx import tsx_templates

def error_page(handler, code: int, message: str = None):
    default_messages = {
        100: "Continue",
        101: "Switching Protocols",
        102: "Processing",
        200: "OK",
        201: "Created",
        202: "Accepted",
        203: "Non-Authoritative Information",
        204: "No Content",
        205: "Reset Content",
        206: "Partial Content",
        300: "Multiple Choices",
        301: "Moved Permanently",
        302: "Found",
        303: "See Other",
        304: "Not Modified",
        305: "Use Proxy",
        307: "Temporary Redirect",
        308: "Permanent Redirect",
        400: "Bad Request",
        401: "Unauthorized",
        402: "Payment Required",
        403: "Forbidden",
        404: "Not Found",
        405: "Method Not Allowed",
        406: "Not Acceptable",
        407: "Proxy Authentication Required",
        408: "Request Timeout",
        409: "Conflict",
        410: "Gone",
        411: "Length Required",
        412: "Precondition Failed",
        413: "Payload Too Large",
        414: "URI Too Long",
        415: "Unsupported Media Type",
        416: "Range Not Satisfiable",
        417: "Expectation Failed",
        418: "I'm a teapot",
        422: "Unprocessable Entity",
        425: "Too Early",
        426: "Upgrade Required",
        428: "Precondition Required",
        429: "Too Many Requests",
        431: "Request Header Fields Too Large",
        451: "Unavailable For Legal Reasons",
        500: "Internal Server Error",
        501: "Not Implemented",
        502: "Bad Gateway",
        503: "Service Unavailable",
        504: "Gateway Timeout",
        505: "HTTP Version Not Supported",
        506: "Variant Also Negotiates",
        507: "Insufficient Storage",
        508: "Loop Detected",
        510: "Not Extended",
        511: "Network Authentication Required"
    }

    msg = message or default_messages.get(code, "Unknown error")

    try:
        handler.send_response(code)
        handler.send_header("Content-Type", "text/html; charset=utf-8")
        handler.end_headers()
        template = error_path
        if template is None or template.endswith(".html"):
            return html_templates.render(template or "error.html", code=code, message=msg)
        elif template.endswith(".tsx"):
            return tsx_templates.render(template, code=code, message=msg)
    except Exception:
        try:
            handler.send_response(500)
            handler.send_header("Content-Type", "text/plain; charset=utf-8")
            handler.end_headers()
            return b"Internal Server Error"
        except:
            pass



import base64

def basic_auth(username: str, password: str):
    expected = base64.b64encode(
        f"{username}:{password}".encode()
    ).decode()

    def middleware(handler):
        auth = handler.headers.get("Authorization")

        if not auth or not auth.startswith("Basic "):
            handler.send_response(401)
            handler.send_header("WWW-Authenticate", 'Basic realm="Boask"')
            handler.end_headers()
            raise StopIteration

        received = auth.split(" ", 1)[1]
        if received != expected:
            handler.send_response(403)
            handler.end_headers()
            raise StopIteration

    return middleware



import urllib.parse
from .users import USERS, hash_password

def register(handler):
    if handler.command != "POST":
        handler.send_response(405)
        handler.end_headers()
        return b"Method Not Allowed"

    length = int(handler.headers.get("Content-Length", 0))
    body = handler.rfile.read(length).decode("utf-8")

    data = urllib.parse.parse_qs(body)
    username = data.get("username", [None])[0]
    password = data.get("password", [None])[0]

    if not username or not password:
        handler.send_response(400)
        handler.end_headers()
        return b"Missing username or password"

    if username in USERS:
        handler.send_response(409)
        handler.end_headers()
        return b"User already exists"

    USERS[username] = hash_password(password)

    handler.send_response(201)
    handler.end_headers()
    return b"User registered"



from setuptools import setup, find_packages
from pathlib import Path

version_file = Path("boask") / "version.py"
version_globals = {}
with open(version_file, "r") as f:
    exec(f.read(), version_globals)

__version__ = version_globals["__version__"]

setup(
    name="boask",
    version=__version__,
    description="Pure Python website engine. Zero dependencies. Real JSX templates.",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="OXOP",
    author_email="minbartek41@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.8",
    license="MIT",
    keywords=["web", "minimal", "python", "no-dependencies"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)



def cors(
    allow_origin="*",
    allow_methods="GET,POST,PUT,DELETE,OPTIONS",
    allow_headers="Content-Type, Authorization"
):
    def middleware(handler):
        handler.send_header("Access-Control-Allow-Origin", allow_origin)
        handler.send_header("Access-Control-Allow-Methods", allow_methods)
        handler.send_header("Access-Control-Allow-Headers", allow_headers)

        if handler.command == "OPTIONS":
            handler.send_response(204)
            handler.end_headers()
            raise StopIteration

    return middleware
